; MOON8 NES project
; Brad Smith, 2012
;
; bank_main.s
;
; Init bank and main menu.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; PAL detection routine

.scope DetectPAL
.include "detect_pal.s"
.endscope

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; graphics data

; PPU binaries
bin_chr:
    .incbin "gfx\tiles.chr"
bin_pal:
    .incbin "gfx\palette.pal"
    
; attribute pages
att_frame:
    .incbin "gfx\frame.atr"
att_info:
    .incbin "gfx\info.atr"
att_player:
    .incbin "gfx\player.atr"
 
; album art (top left: 11,3)
cover_row0: .byte $09, $18, $18, $18, $18, $18, $18, $18, $18, $0A
cover_row1: .byte $07, $00, $00, $00, $00, $00, $00, $00, $00, $08
cover_row2: .byte $07, $00, $00, $00, $00, $12, $00, $00, $00, $08
cover_row3: .byte $07, $00, $00, $13, $14, $15, $00, $00, $00, $08
cover_row4: .byte $07, $01, $02, $03, $04, $05, $06, $00, $00, $08
cover_row5 = cover_row1
cover_row6 = cover_row1
cover_row7 = cover_row1
cover_row8 = cover_row1
cover_row9: .byte $19, $17, $17, $17, $17, $17, $17, $17, $17, $1A

cover_sprites: ; (rainbow) top left is 128, 50
    .byte 49 +  0, $B0, %00000010, 128 +  0
    .byte 49 +  0, $B1, %00000010, 128 +  8
    .byte 49 +  0, $B2, %00000010, 128 + 16
    .byte 49 +  0, $B3, %00000010, 128 + 24
    .byte 49 +  0, $B4, %00000011, 128 +  0
    .byte 49 +  0, $B5, %00000011, 128 +  8
    .byte 49 +  0, $B6, %00000011, 128 + 16
    .byte 49 +  8, $C2, %00000010, 128 + 16
    .byte 49 +  8, $C3, %00000010, 128 + 24
    .byte 49 +  8, $C5, %00000011, 128 +  8
    .byte 49 +  8, $C6, %00000011, 128 + 16
    .byte 49 +  8, $C7, %00000011, 128 + 24
caret_sprite:
    .byte     255, $E0, %00000010,      255

CARET_X = (12*4)+3
CARET_Y = (12*4)+0

; centred MOON8 text (left: 13,14)
text_moon8: .byte $D1, $D2, $D3, $D4, $D5, $D6

; title text
text_title0: .asciiz "PLAY"
text_title1: .asciiz "INFO"

; track text (top left: ?, 16)
text_track0: .asciiz "Speak to Me"
text_track1: .asciiz "Breathe"
text_track2: .asciiz "On the Run"
text_track3: .asciiz "Time"
text_track4: .asciiz "The Great Gig in the Sky"
text_track5: .asciiz "Money"
text_track6: .asciiz "Us and Them"
text_track7: .asciiz "Any Colour You Like"
text_track8: .asciiz "Brain Damage"
text_track9: .asciiz "Eclipse"

; info page
text_info: ; 01234567890123456789012345678912
    .byte      "MOON8: a cover of Pink    " ; 0
    .byte      "Floyd's Dark Side of the  " ; 1
    .byte      "Moon, by Brad Smith, 2010." ; 2
    .byte      "                          " ; 3
    .byte      "Original music @ Hampshire" ; 4
    .byte      "House Publishing, 1973.   " ; 5
    .byte      "                          " ; 6
    .byte      "     A - continuous play  " ; 7
    .byte      "     B - single play      " ; 8
    .byte      "   </> - slow / fast      " ; 9
    .byte      "   {/} - select track     " ; 0
    .byte      " START - pause            " ; 1
    .byte      "SELECT - return to menu   " ; 2
    .byte      "                          " ; 3
    .byte      "Famitracker code by       " ; 4
    .byte      "Jonathan Liss. Music,     " ; 5
    .byte      "images, and other code by " ; 6
    .byte      "Brad Smith, 2012.         " ; 7
    .byte      "                          " ; 8
    .byte      "Many thanks to all at     " ; 9
    .byte      "NESDev, RetroUSB, and     " ; 0
    .byte      "thank YOU for listening!  " ; 1
    .byte      "                          " ; 2
    .byte      "http://rainwarrior.ca     " ; 3
 
 text_build:
    .byte BUILD_NUMBER
 
 text_ntsc:
    .byte "NTSC"
 text_pal:
    .byte "PAL "

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; ppu routines

; ZP scratch/argument variables used by ppu routines
ZP_PPU_LOAD  = zp_base + 0 ; word
ZP_PPU_ADDR  = zp_base + 2 ; word
ZP_PPU_COUNT = zp_base + 4 ; byte

ppu_off:
    LDA #0
    STA PPU_MASK ; disable render
    STA PPU_CTRL ; disable NMI
    RTS

ppu_on:
    LDA #%10000000 ; enable NMI
    STA PPU_CTRL
    RTS

ppu_setup:
;
;   loads CHR-RAM and palette
;
    LDA #2 ; two 4k pages
    STA ZP_PPU_COUNT
    ; select CHR ROM memory
    LDA PPU_STAT ; reset latch
    PPU_LATCH $0000
@load_chr_4k:
    ZP_ADDR bin_chr, ZP_PPU_LOAD
    LDX #16 ; pages of 256 bytes to load
    LDY #0
@load_chr_loop:
    ; load 256 bytes
    LDA (ZP_PPU_LOAD), Y
    STA PPU_DATA
    INY
    BNE @load_chr_loop
    ; next page
    INC ZP_PPU_LOAD + 1
    DEX
    BNE @load_chr_loop
    ; sprite and tile use the same 4k CHR ROM
    DEC ZP_PPU_COUNT
    BNE @load_chr_4k
    ; finished loading CHR
    ;
    ; load palette
    PPU_LATCH $3F00
    LDA #2
    STA ZP_PPU_COUNT
@load_pal_twice:
    LDX #16
    LDY #0
@load_pal_loop:
    LDA bin_pal, Y
    STA PPU_DATA
    INY
    DEX
    BNE @load_pal_loop
    DEC ZP_PPU_COUNT
    BNE @load_pal_twice
    ; finished loading palette
    RTS

ppu_update:
;
; called by NMI to update nametable in vblank
;
; bss_ppu = 256 byte PPU update stream
; data is in stream of form:
;   A B C [...]
;   A = number of bytes of data (A = 0 means no more data)
;   B, C = high, low PPU address to begin write
;   [...] = A bytes of data
; Be careful not to attempt more than 256 bytes of data (counter will loop)
;
    LDA PPU_STAT ; reset latch
    LDY #0
@packet_loop:
    LDA bss_ppu, Y
    BEQ @load_finished
    TAX
    INY
    LDA bss_ppu, Y
    STA PPU_ADDR
    INY
    LDA bss_ppu, Y
    STA PPU_ADDR
    INY
@fill_loop:
    LDA bss_ppu, Y
    STA PPU_DATA
    INY
    DEX
    BNE @fill_loop
    JMP @packet_loop
@load_finished:
    RTS

ppu_att:
;
;   loads an attribute table
;   ZP_PPU_LOAD = attribute table address
;
    LDA PPU_STAT
    PPU_LATCH $23C0
    LDX #$40
    LDY #0
@att_loop:
    LDA (ZP_PPU_LOAD), Y
    STA PPU_DATA
    INY
    DEX
    BNE @att_loop
    RTS

ppu_cls:
;
;   clears the screen
;
    LDA PPU_STAT
    PPU_LATCH $2000
    LDA #0
    LDY #30
@loop_y:    
    LDX #32
@loop_x:
    STA PPU_DATA
    DEX
    BNE @loop_x
    DEY
    BNE @loop_y
    ; reset bss_ppu buffer
    LDA #0
    LDX #0
@loop_ppu_buffer:
    STA bss_ppu, X
    INX
    BNE @loop_ppu_buffer
    RTS

ppu_store:
;
;   store row of data to current PPU address
;   ZP_PPU_LOAD = row of data
;   X = bytes to write
;   see PPU_STORE macro
;
    LDY #0
@store_loop:
    LDA (ZP_PPU_LOAD), Y
    STA PPU_DATA
    INY
    DEX
    BNE @store_loop
    RTS
    
ppu_string:
;
;   same as ppu_store but stops on 0 instead of using X for length
;   see PPU_STRING macro
;
    LDY #0
@string_loop:
    LDA (ZP_PPU_LOAD), Y
    BEQ @string_end
    STA PPU_DATA
    INY
    DEX
    JMP @string_loop
@string_end:
    RTS
    
oam_cls:
;
;   clears sprites
;
    LDX #0
    LDA #$FF
@oam_cls_loop:
    STA bss_oam, X
    DEX
    BNE @oam_cls_loop
    RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; menu backgrounds

draw_frame:
    JSR ppu_cls
    ZP_ADDR att_frame, ZP_PPU_LOAD
    JSR ppu_att
    PPU_STORE 11,  3, 10, cover_row0
    PPU_STORE 11,  4, 10, cover_row1
    PPU_STORE 11,  5, 10, cover_row2
    PPU_STORE 11,  6, 10, cover_row3
    PPU_STORE 11,  7, 10, cover_row4
    PPU_STORE 11,  8, 10, cover_row5
    PPU_STORE 11,  9, 10, cover_row6
    PPU_STORE 11, 10, 10, cover_row7
    PPU_STORE 11, 11, 10, cover_row8
    PPU_STORE 11, 12, 10, cover_row9
    JSR oam_cls
    LDX #((12+1)*4) ; +1 includes caret sprite offscreen
@draw_rainbow:
    DEX
    LDA cover_sprites, X
    STA bss_oam, X
    CPX #0
    BNE @draw_rainbow
    RTS

draw_page_title:
    JSR draw_frame
    PPU_STORE 13, 14, 6, text_moon8
    PPU_STRING 14, 18, text_title0
    PPU_STRING 14, 20, text_title1
    RTS

draw_page_track:
    JSR draw_frame
    PPU_STORE 13, 14, 6, text_moon8
    PPU_STRING 4, 16, text_track0
    PPU_STRING 4, 17, text_track1
    PPU_STRING 4, 18, text_track2
    PPU_STRING 4, 19, text_track3
    PPU_STRING 4, 20, text_track4
    PPU_STRING 4, 21, text_track5
    PPU_STRING 4, 22, text_track6
    PPU_STRING 4, 23, text_track7
    PPU_STRING 4, 24, text_track8
    PPU_STRING 4, 25, text_track9
    RTS

draw_page_info:
    JSR ppu_cls
    ZP_ADDR att_info, ZP_PPU_LOAD
    JSR ppu_att
    JSR oam_cls
    .repeat 24, I
        PPU_STORE 3, (3+I), 26, (text_info+(26*I))
    .endrepeat
    PPU_STORE 1, 28, 3, text_build
    LDA zp_pal
    BNE @draw_info_pal
    PPU_STORE 5, 28, 4, text_ntsc
    JMP @draw_info_finished
@draw_info_pal:
    PPU_STORE 5, 28, 4, text_pal
@draw_info_finished:
    RTS

draw_page_play:
    JSR ppu_cls
    JSR draw_frame
    ZP_ADDR att_player, ZP_PPU_LOAD
    JSR ppu_att
    RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; menu music playback

.include "ft_driver.s"

menu_music:
    .incbin "music/main.bin"

; play menu sounds
    
play_silent:
    ;LDA #TUNE_TEST
    LDA #TUNE_SILENT
    STA zp_play_tune
    RTS

play_move:
    LDA #TUNE_MOVE
    STA zp_play_tune
    RTS

play_select:
    LDA #TUNE_SELECT
    STA zp_play_tune
    RTS

; call once per frame

play_tick:
    LDA zp_play_tune
    CMP #PLAY_NONE
    BEQ @play_none
    CMP #PLAY_CONTINUE
    BEQ @play_continue
    ; new song selected in A, initialize it
    LDX zp_pal
    JSR Famitracker::ft_music_init
    STB PLAY_CONTINUE, zp_play_tune
@play_continue:
    ; tick the playing track
    JSR Famitracker::ft_music_play
@play_none:
    RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; menu title page

page_title_setup:
    JSR draw_page_title
    ; position caret horizontally
    LDA #((13*8)-2)
    STA bss_oam + CARET_X
    RTS

page_title_tick:
    ; respond to controller input
    ;
        LDA zp_gamepad_press
        AND #(GP_A | GP_B | GP_START)
        BEQ :+
        JSR play_select
        LDA zp_title_option
        BNE @switch_info
    @switch_track:
        JSR ppu_off
        STB MENU_TRACK, zp_main_menu
        JSR page_track_setup
        JSR ppu_on
        RTS
    @switch_info:
        JSR ppu_off
        STB MENU_INFO, zp_main_menu
        JSR page_info_setup
        JSR ppu_on
        RTS
    :
        LDA zp_gamepad_press
        AND #(GP_U | GP_D | GP_SELECT)
        BEQ :+
        JSR play_move
        INC zp_title_option
        LDA zp_title_option
        AND #1
        STA zp_title_option
    :
    ; position caret vertically
    LDA zp_title_option
    ASL ; * 16
    ASL
    ASL
    ASL
    CLC
    ADC #((18*8)-1)
    STA bss_oam + CARET_Y
    ; finished tick
    RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; menu track page

page_track_setup:
    JSR draw_page_track
    ; position caret horizontally
    LDA #(( 3*8)-2)
    STA bss_oam + CARET_X
    RTS

play_track:
    JSR ppu_off
    JSR draw_page_play
    ; reset timer
    LDA #0
    STA zp_time + 0
    STA zp_time + 1
    STA zp_time + 2
    STA zp_time + 3
    STA zp_time + 4
    ; swap to appropriate song bank
    LDX zp_song
    LDY song_table_bank, X
    JMP bank_swap
    
page_track_tick:
    ;
        LDA zp_gamepad_press
        AND #(GP_A | GP_START)
        BEQ :+
        STB 0, zp_single ; continuous play
        STB 1, zp_switch ; signal bankswitch
        RTS
    :
        LDA zp_gamepad_press
        AND #(GP_B)
        BEQ :+
        STB 1, zp_single ; single play
        STB 1, zp_switch ; signal bankswitch
        RTS
    :
        LDA zp_gamepad_press
        AND #(GP_SELECT)
        BEQ :+
        JSR ppu_off
        STB MENU_TITLE, zp_main_menu
        JSR page_title_setup
        JSR ppu_on
        RTS
    :
        LDA zp_gamepad_press
        AND #(GP_U)
        BEQ :+
        JSR play_move
        DEC zp_song
        BPL :+ ; if song < 0, wrap to 9
        STB 9, zp_song
    :
        LDA zp_gamepad_press
        AND #(GP_D)
        BEQ :+
        JSR play_move
        INC zp_song
        LDA zp_song
        CMP #10
        BCC :+ ; if song > 9, wrap to 0
        STB 0, zp_song
    :
    ; position caret vertically
    LDA zp_song
    ASL ; * 8
    ASL
    ASL
    CLC
    ADC #((16*8)-1)
    STA bss_oam + CARET_Y
    ; finished tick
    RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; menu info page

page_info_setup:
    JSR draw_page_info
    RTS

page_info_tick:
    ; respond to controller input
    ;
        LDA zp_gamepad_press
        AND #(GP_A | GP_B | GP_SELECT | GP_START)
        BEQ :+
        JSR ppu_off
        STB MENU_TITLE, zp_main_menu
        JSR page_title_setup
        JSR ppu_on
        RTS
    :
    ; finished tick
    RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; controller read

; store three controller reads temporarily
ZP_GAMEPAD_TEMP = zp_base

gamepad_read:
;
;   safely reads first gamepad state
;
    ; read controller 3 times to avoid DPCM bus conflict
    .repeat 3, I
        ; strobe to latch data
        LDA #1
        STA GAMEPAD0
        LDA #0
        STA GAMEPAD0
        ; store bit to test when all bits have been read
        LDA #%10000000
        STA ZP_GAMEPAD_TEMP + I
        ;
    :
        LDA GAMEPAD0
        ROR
        ROR ZP_GAMEPAD_TEMP + I
        BCC :-
    .endrepeat
    ; at least two reads should be the same, put the 2 of 3 result in A
    LDA ZP_GAMEPAD_TEMP + 0
    CMP ZP_GAMEPAD_TEMP + 1
    BEQ @gamepad_return ; A = 0 + 1
    CMP ZP_GAMEPAD_TEMP + 2
    BEQ @gamepad_return ; A = 0 + 2 (1 is different)
    LDA ZP_GAMEPAD_TEMP + 1
    ;                   ; A = 1 (presume 1 = 2)
@gamepad_return:
    RTS
    
gamepad_pressed:
;
;   updates zero page gamepad variables:
;     zp_gamepad       = current state
;     zp_gamepad_last  = last frame
;     zp_gamepad_press = new pressed buttons this frame
;
    ; remember previous frame
    LDA zp_gamepad
    STA zp_gamepad_last
    ; read current frame
    JSR gamepad_read
    STA zp_gamepad
    ; determine new presses
    LDA zp_gamepad_last
    EOR #%11111111 ; A = gamepad not pressed last frame
    AND zp_gamepad ; A = gamepad pressed this frame
    STA zp_gamepad_press
    RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; main code

bank_init:
    ; only perform initialization the first time
    LDA zp_main_init
    BNE initialized
    ;
    ; basic startup
    SEI       ; set interrupt flag
    CLD       ; disable decimal mode
    LDX #$40
    STX $4017 ; disable frame IRQ
    LDX #$FF
    TXS       ; set up stack
    INX       ; X = 0
    STX PPU_CTRL ; disable NMI
    STX PPU_MASK ; disable render
    STX $4010 ; disable DPCM IRQ
    ;
    ; PPU 2-frame warm up and memory clear
    BIT PPU_STAT
@init_vblank_1:
    BIT PPU_STAT
    BPL @init_vblank_1
    TXA
@init_clear:
    STA $000, X
    STA $100, X
    STA $200, X
    STA $300, X
    STA $400, X
    STA $500, X
    STA $600, X
    STA $700, X
    INX
    BNE @init_clear
@init_vblank_2:
    BIT PPU_STAT
    BPL @init_vblank_2
    ;
    ; initialize variables
    STB PLAY_NONE, zp_play_tune
    ;
    ; detect PAL
    JSR ppu_setup ; wipe screen
    JSR ppu_cls
    JSR oam_cls
    LDA #%10000000 ; enable NMI
    STA PPU_CTRL
    JSR DetectPAL::getTVSystem
    CMP #0
    BEQ @ntsc
    ; PAL
    STB 1, zp_pal
    STB 50, zp_fps
    JMP @init_end
@ntsc:
    STB 0, zp_pal
    STB 60, zp_fps
    ;
    ; init finished
    ;
@init_end:
    STB 1, zp_main_init
initialized:
    JSR ppu_off
    JSR ppu_setup
    ; setup famitracker data
    ZP_ADDR menu_music, ft_music_addr
    ;
    ; setup menu
    ;
@setup_menu:
    LDA zp_main_menu
    ;
        CMP #MENU_TITLE
        BNE :+
        JSR page_title_setup
        JMP @menu_ready
    :
        CMP #MENU_TRACK
        BNE :+
        JSR page_track_setup
        JMP @menu_ready
    :
        CMP #MENU_INFO
        BNE :+
        JSR page_info_setup
        JMP @menu_ready
    :
    ; error, unknown menu page, should not reach this point
    STB 0, zp_main_menu
    JMP @setup_menu
@menu_ready:
    ; enable NMI and enter main loop
    JSR ppu_on
    ;
    ;
    ; main loop executes just after NMI
    ;
    ;
main_loop:
    LDA zp_frame
@wait_nmi:
    CMP zp_frame
    BEQ @wait_nmi
    ; read gamepad
    JSR gamepad_pressed
    ; play sound
    JSR play_tick
    ; tick menu
    LDA zp_main_menu
    ;
        CMP #MENU_TITLE
        BNE :+
        JSR page_title_tick
        JMP @menu_ticked
    :
        CMP #MENU_TRACK
        BNE :+
        JSR page_track_tick
        JMP @menu_ticked
    :
        CMP #MENU_INFO
        BNE :+
        JSR page_info_tick
        JMP @menu_ticked
    :
    ; error, unknown menu page, should not reach this point
    STB 0, zp_main_menu
    JMP bank_init
    ;
@menu_ticked:
    ; if zp_switch is flaggged, jump to play_track
    LDA zp_switch
    BEQ @no_switch
    STB 0, zp_switch
    JMP play_track
@no_switch:
    ; mark frame ready
    STB 1, zp_frame_ready
    ; next frame
    .if PROFILE
        LDA #%00011111
        STA PPU_MASK
    .endif
    JMP main_loop

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; NMI rendering routine

bank_nmi:
    PHA
    TXA
    PHA
    TYA
    PHA
    ; if new data for PPU, load it
    LDA zp_frame_ready
    BEQ @done_ppu_update
    ; upload sprite OAM via DMA
    LDA #0
    STA OAM_ADDR
    LDA #>bss_oam
    STA OAM_DMA
    ; update ppu memory if requested
    JSR ppu_update
    STB 0, zp_frame_ready
    ; enable render
    LDA #%00011110
    STA PPU_MASK
@done_ppu_update:
    LDA PPU_STAT
    ; reset scroll
    LDA #0
    STA PPU_SCRL
    STA PPU_SCRL
    INC zp_frame
    PLA
    TAY
    PLA
    TAX
    PLA
    RTI

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; IRQ (not used)

bank_irq:
    RTI

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; NSF

.ifdef NSF

nsf_init:
;
;   X = NTSC/PAL
;   A = track
;
    STX zp_pal
    PHA
    ZP_ADDR menu_music, ft_music_addr
    LDA #TUNE_SILENT
    JSR Famitracker::ft_music_init
    PLA
    ; A = song number
    ;
    ;   0 = side one
        CMP #0
        BNE :+
        STB 0, zp_single ; continuous play
        STB 0, zp_song
        JMP @track_chosen
    : ; 1 = side two
        CMP #1
        BNE :+
        LDA #5
        STB 0, zp_single ; continuous play
        STB 5, zp_song
        JMP @track_chosen
    :
    SEC
    SBC #2
    STA zp_song
    STB 1, zp_single ; single play
@track_chosen:
    LDA zp_song
    TAX
    LDA song_table_bank, X 
    JMP nsf_bank_swap

nsf_bank_init:
    ; this init should not be used
    RTS

nsf_bank_play:
    ; this play should not be used
    RTS

.endif

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; end of file
