; MOON8 NES project
; Brad Smith, 2012
;
; bank_stub.s
;
; AxROM bank swap stub. Provides a bankswitching mechanism
; as well as ensuring proper initialization from any bank.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; global data

; bank and index for all songs
;
; references banks set up in moon8.s
;   song_table_bank = bank to use
;   song_table_index = music label (0-3)
;
song_table_bank:
    .byte 1, 2, 1, 3, 2, 4, 5, 6, 6, 7
song_table_index:
    .byte 0, 0, 1, 0, 1, 0, 0, 0, 1, 0

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; initialization code

stub_init:
    SEI              ; set interrupt flag
    LDY #$00         ; select bank 0 (see below)
    STY $2000        ; disable NMI
    STY zp_main_init ; reset initialized flag
    ; fall through to bank_swap

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; AxROM bank swap

; usage:
;   LDY bank
;   JMP bank_swap
;
; bank_swap will JMP to bank_init, which must use RAM or
; register to determine where to go from there. Note that
; this also resets the single-page mirroring to nametable 0.

bank_swap:
    PHA
    TYA
    STA bank_table, Y
    PLA
    JMP bank_init

; bank numbers 0-7 used to prevent bus conflict
bank_table:
.byte $00, $01, $02, $03, $04, $05, $06, $07

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; NSF stub

.ifdef NSF

nsf_play:
    JMP nsf_bank_play

nsf_bank_swap:
    ASL
    ASL
    ASL
    STA $5FF8
    CLC
    .repeat 7, I
        ADC #1
        STA $5FF9 + I
    .endrepeat
    JMP nsf_bank_init

.endif

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; end of file
