; MOON8 NES project
; Brad Smith, 2012
;
; global.s
;
; Global defines, and memory usage.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; global constants

.define BUILD_NUMBER "005"

; PROFILE = 1 turns the screen grey when finished logic
.define PROFILE 0

; common registers
PPU_CTRL = $2000
PPU_MASK = $2001
PPU_STAT = $2002
OAM_ADDR = $2003
OAM_DATA = $2004
PPU_SCRL = $2005
PPU_ADDR = $2006
PPU_DATA = $2007
OAM_DMA  = $4014
GAMEPAD0 = $4016
GAMEPAD1 = $4017

; for zp_play_tune
PLAY_NONE      = $FF          ; don't tick famitracker
PLAY_CONTINUE  = $FE          ; tick famitracker, not starting a new song
; tunes from main.bin
TUNE_TEST      = $00
TUNE_SILENT    = $01
TUNE_MOVE      = $02
TUNE_SELECT    = $03
; for zp_play_mode
PLAY_NORMAL    = $00
PLAY_SLOW      = $01
PLAY_FAST      = $02

; for zp_gamepad
GP_A      = %00000001
GP_B      = %00000010
GP_SELECT = %00000100
GP_START  = %00001000
GP_U      = %00010000
GP_D      = %00100000
GP_L      = %01000000
GP_R      = %10000000

; for zp_main_menu
MENU_TITLE = 0
MENU_TRACK = 1
MENU_INFO  = 2

; sprite tiles
TILE_VU0      = $80
TILE_VU1      = $90
TILE_VU2      = $A0
TILE_DUTY     = $88
TILE_TRIANGLE = $98
TILE_NOISE    = $A8
TILE_DPCM     = $9A
TILE_SINGLE   = $9C
TILE_PAUSE    = $E1

;
; handy macros
;

.macro STB           val, addr
;
;   store immediate byte
;
    LDA #val
    STA addr
.endmacro

.macro ZP_ADDR       addr, zp_var
;
;   place address word on zero page variable
;
    LDA #<addr
    STA zp_var + 0
    LDA #>addr
    STA zp_var + 1
.endmacro

.macro PPU_LATCH     addr
;
;   latch PPU to a given address
;
    LDA #>addr
    STA PPU_ADDR
    LDA #<addr
    STA PPU_ADDR
.endmacro

.macro PPU_TILE      tx, ty
;
;   latch PPU to a given x,y tile
;
    LDA #>($2000+tx+(ty*32))
    STA PPU_ADDR
    LDA #<($2000+tx+(ty*32))
    STA PPU_ADDR
.endmacro

.macro PPU_STORE    tx, ty, count, addr
;
;   easy version of ppu_store
;   loads count bytes from addr into tile tx,ty
;
    ZP_ADDR addr, ZP_PPU_LOAD
    LDA PPU_STAT
    PPU_TILE tx, ty
    LDX #count
    JSR ppu_store
.endmacro

.macro PPU_STRING    tx, ty, addr
;
;   easy version of ppu_string
;   loads string from addr into tile tx,ty
;
    ZP_ADDR addr, ZP_PPU_LOAD
    LDA PPU_STAT
    PPU_TILE tx, ty
    JSR ppu_string
.endmacro
    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; zero page

.segment "ZEROPAGE"

zp_base:           .res 16      ; scratch space on zero page
zp_frame:          .res 1       ; NMI increments every frame
zp_main_init:      .res 1       ; flag if program is initialized
zp_main_menu:      .res 1       ; current menu page
zp_switch:         .res 1       ; signal bankswitch
zp_title_option:   .res 1       ; current title page option
zp_song:           .res 1       ; selected song
zp_gamepad:        .res 1       ; gamepad state
zp_gamepad_last:   .res 1       ; last gamepad state
zp_gamepad_press:  .res 1       ; gamepad pressed this frame
zp_frame_skip:     .res 1       ; frame skip
zp_frame_ready:    .res 1       ; set to 1 when new data should be sent to PPU
zp_pal:            .res 1       ; PAL flag
zp_fps:            .res 1       ; 60/50 NTSC/PAL
zp_time:           .res 5       ; 4 digit clock + frame counter
zp_single:         .res 1       ; 0 = continuous play, 1 = single play
zp_play_tune:      .res 1       ; set to a number other than PLAY_NONE to play this song next NMI
zp_play_mode:      .res 1       ; normal / slow / fast playback
zp_play_done:      .res 1       ; 1 = stopped playback
zp_noise_anim:     .res 2       ; animated noise (0 = current pos, 1 = sub-anim)
zp_dpcm_anim:      .res 1       ; animated dpcm (counts down from 15 on DPCM trigger)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; RAM

.segment "BSS"

bss_oam:       .res 256     ; sprite OAM data
bss_ppu:       .res 256     ; PPU update buffer
ft_music_addr: .res 2       ; address of music data

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Famitracker globals, moved from ft_driver/driver.s

; Famitracker global contants

USE_BANKSWITCH = 0		; Enable bankswitching code
USE_DPCM = 1			; Enable DPCM channel (currently broken, leave enabled to avoid trouble).
						; Also leave enabled for expansion chips
NTSC_PERIOD_TABLE = 1	; Enable this to include the NTSC period table
PAL_PERIOD_TABLE = 1	; Enable this to include the PAL period table
ENABLE_ROW_SKIP = 1		; Enable this to add code for seeking to a row > 0 when using skip command

TUNE_PATTERN_LENGTH		= $00
TUNE_FRAME_LIST_POINTER	= $01

.ifdef USE_DPCM
	CHANNELS	 = 5
	DPCM_CHANNEL = 4
	WAVE_CHANS 	 = CHANNELS - 1
.else
	CHANNELS	 = 4
	WAVE_CHANS 	 = CHANNELS
.endif

EFF_CHANS = CHANNELS - 1		; # of channels using vibrato & arpeggio effects (not used by DPCM)
NOISE_CHANNEL = 3
SFX_CHANS		= CHANNELS
SFX_WAVE_CHANS 	= WAVE_CHANS
HEAD_SPEED				= 11
HEAD_TEMPO				= 12

EFF_NONE				= 0
EFF_ARPEGGIO			= 1
EFF_PORTAMENTO			= 2
EFF_PORTA_UP			= 3
EFF_PORTA_DOWN			= 4
EFF_SLIDE_UP_LOAD		= 5
EFF_SLIDE_UP			= 6
EFF_SLIDE_DOWN_LOAD		= 7
EFF_SLIDE_DOWN			= 8

; Famitracker ZP variables

.segment "ZEROPAGE"

var_Temp:				.res 1						; Temporary 8-bit
var_Temp2:				.res 1
var_Temp3:				.res 1
var_Temp4:				.res 1
var_Temp16:				.res 2						; Temporary 16-bit
var_Temp_Pointer:		.res 2						; Temporary
var_Temp_Pointer2:		.res 2
var_Temp_Pattern:		.res 2						; Pattern address (temporary)
var_Note_Table:			.res 2
var_Temp_Inst:			.res 1
ACC:					.res 2						; Used by division routine
AUX:					.res 2
EXT:					.res 2
last_zp_var:			.res 1						; Not used

; Famitracker BSS

.segment "BSS"

; Song header (necessary to be in order)
var_Song_list:			.res 2						; Song list address
var_Instrument_list:	.res 2						; Instrument list address
.ifdef USE_DPCM
var_dpcm_inst_list:		.res 2						; DPCM instruments
var_dpcm_pointers:		.res 2						; DPCM sample pointers
.endif
var_SongFlags:			.res 1						; Song flags, bit 0 = bankswitched, bit 1 = old vibrato, bit 2 - 7 = unused

var_Channels:			.res 1						; Channel enable/disable
	
; Track header (necessary to be in order)
var_Frame_List:			.res 2						; Pattern list address
var_Frame_Count:		.res 1						; Number of frames
var_Pattern_Length:		.res 1						; Global pattern length
var_Speed:				.res 1						; Speed setting
var_Tempo:				.res 1						; Tempo setting
var_InitialBank:		.res 1

; General
var_PlayerFlags:		.res 1						; Player flags, bit 0 = playing, bit 1 - 7 unused
var_Pattern_Pos:		.res 1						; Global pattern row
var_Current_Frame:		.res 1						; Current frame
var_Load_Frame:			.res 1						; 1 if new frame should be loaded

var_Tempo_Accum:		.res 2						; Variables for speed division
var_Tempo_Count:		.res 2						;  (if tempo support is not needed then this can be optimized)
var_Tempo_Dec:			.res 2
var_VolTemp:			.res 1						; So the Exx command will work
var_Sweep:				.res 1						; This has to be saved

.ifdef USE_BANKSWITCH
var_Bank:				.res 1
.endif
var_Jump:				.res 1						; If a Jump should be executed
var_Skip:				.res 1						; If a Skip should be executed
.ifdef ENABLE_ROW_SKIP
var_SkipTo:				.res 1						; Skip to row number
.endif

var_sequence_ptr:		.res 1
var_sequence_result:	.res 1

;var_enabled_channels:	.res 1
; Channel variables
; General channel variables, used by the pattern reader (all channels)
var_ch_PatternAddrLo:	.res CHANNELS				; Holds current pattern position
var_ch_PatternAddrHi:	.res CHANNELS
.ifdef USE_BANKSWITCH
var_ch_Bank:			.res CHANNELS				; Pattern bank
.endif
var_ch_Note:			.res CHANNELS				; Current channel note
var_ch_VolColumn:		.res CHANNELS				; Volume column
var_ch_Delay:			.res CHANNELS				; Delay command
var_ch_NoteCut:			.res CHANNELS
var_ch_State:			.res CHANNELS
var_ch_FinePitch:		.res CHANNELS				; Fine pitch setting

var_ch_NoteDelay:		.res CHANNELS				; Delay in rows until next note
var_ch_DefaultDelay:	.res CHANNELS				; Default row delay, if exists

; Following is specific to chip channels (2A03, VRC6...)
var_ch_TimerPeriodHi:	.res EFF_CHANS				; Current channel note period
var_ch_TimerPeriodLo:	.res EFF_CHANS
var_ch_PeriodCalcLo:	.res EFF_CHANS 				; Frequency after fine pitch and vibrato has been applied
var_ch_PeriodCalcHi:	.res EFF_CHANS
var_ch_OutVolume:		.res CHANNELS				; Volume for the APU
var_ch_VolSlide:		.res CHANNELS				; Volume slide

; --- Testing ---
;var_ch_LoopCounter:		.res CHANNELS
; --- Testing --- 

; Square 1 & 2 variables
var_ch_Sweep:			.res 2						; Hardware sweep
var_ch_PrevFreqHigh:	.res 2						; Used only by 2A03 pulse channels

; Sequence variables
var_ch_SeqVolume:		.res SFX_WAVE_CHANS * 2		; Sequence 1: Volume
var_ch_SeqArpeggio:		.res SFX_WAVE_CHANS * 2		; Sequence 2: Arpeggio
var_ch_SeqPitch:		.res SFX_WAVE_CHANS * 2		; Sequence 3: Pitch bend
var_ch_SeqHiPitch:		.res SFX_WAVE_CHANS * 2		; Sequence 4: High speed pitch bend
var_ch_SeqDutyCycle:	.res SFX_WAVE_CHANS * 2		; Sequence 5: Duty cycle / Noise Mode
var_ch_Volume:			.res SFX_WAVE_CHANS			; Output volume
var_ch_DutyCycle:		.res SFX_WAVE_CHANS			; Duty cycle / Noise mode
var_ch_SequencePtr1:	.res SFX_WAVE_CHANS			; Index pointers for sequences
var_ch_SequencePtr2:	.res SFX_WAVE_CHANS
var_ch_SequencePtr3:	.res SFX_WAVE_CHANS
var_ch_SequencePtr4:	.res SFX_WAVE_CHANS
var_ch_SequencePtr5:	.res SFX_WAVE_CHANS

; Track variables for effects
var_ch_Effect:			.res EFF_CHANS				; Arpeggio & portamento
var_ch_EffParam:		.res EFF_CHANS				; Effect parameter (used by portamento and arpeggio)

var_ch_PortaToHi:		.res EFF_CHANS 				; Portamento
var_ch_PortaToLo:		.res EFF_CHANS
var_ch_ArpeggioCycle:	.res EFF_CHANS				; Arpeggio cycle

var_ch_VibratoPos:		.res EFF_CHANS				; Vibrato
var_ch_VibratoDepth:	.res EFF_CHANS		
var_ch_VibratoSpeed:	.res EFF_CHANS
var_ch_TremoloPos:		.res EFF_CHANS				; Tremolo
var_ch_TremoloDepth:	.res EFF_CHANS				; combine these
var_ch_TremoloSpeed:	.res EFF_CHANS

;var_ch_VibratoParam:	.res EFF_CHANS
;var_ch_TremoloParam:	.res EFF_CHANS

; DPCM variables
.ifdef USE_DPCM
var_ch_SamplePtr:		.res 1						; DPCM sample pointer
var_ch_SampleLen:		.res 1						; DPCM sample length
var_ch_SamplePitch:		.res 1						; DPCM sample pitch
var_ch_DPCMDAC:			.res 1						; DPCM delta counter setting
var_ch_DPCM_Offset:		.res 1	
var_ch_DPCM_Retrig:		.res 1						; DPCM retrigger
var_ch_DPCM_RetrigCntr:	.res 1
var_ch_DPCM_EffPitch:	.res 1
.endif

; End of variable space
last_bss_var:			.res 1						; Not used

; Famitracker macro moved from ft_driver/instrument.s

.macro load_inst seq_addr, seq_ptr

	ror var_Temp3
	bcc	:++
	clc
	lda (var_Temp_Pointer), y
	adc ft_music_addr
	sta var_Temp16
	iny
	lda (var_Temp_Pointer), y
	adc ft_music_addr + 1
	sta var_Temp16 + 1
	iny

	lda var_Temp16
	cmp seq_addr, x
	bne :+
	lda var_Temp16 + 1
	cmp seq_addr + SFX_WAVE_CHANS, x
;	bne :+

	; Both equal

	jmp :+++

:	lda var_Temp16
	sta seq_addr, x
	lda var_Temp16 + 1
	sta seq_addr + SFX_WAVE_CHANS, x

	lda #$00
	sta seq_ptr, x

	jmp :++		; branch always
:	lda #$00
	sta seq_addr, x
	sta seq_addr + SFX_WAVE_CHANS, x
:
.endmacro

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; end of file
